import 'dart:convert';

import 'package:flutter/painting.dart';
import 'package:furman_now/src/services/days_of_week.dart';
import 'package:latlong2/latlong.dart';
import 'package:http/http.dart' as http;
import 'package:palette_generator/palette_generator.dart';

class RestaurantService {
  static Future<List<Restaurant>> fetchRestaurants() async {
    var restaurants = (await _fetchRestaurants()).toList();
    await _fetchRestaurantHours(restaurants);
    return restaurants;
  }

  static Future<Iterable<Restaurant>> _fetchRestaurants() async {
    final response = await http
      .get(Uri.parse("https://cs.furman.edu/~csdaemon/FUNow/restaurantGet.php"));

    if (response.statusCode == 200) {
      // If the server did return a 200 OK response,
      // then parse the JSON.
      final restaurantsJson = jsonDecode(response.body);
      return (restaurantsJson["results"] as List<dynamic>).map((restaurant) =>
          Restaurant.fromJson(restaurant)
      );
    } else {
      // If the server did not return a 200 OK response,
      // then throw an exception.
      throw Exception('Failed to load restaurants.');
    }
  }

  static Future<void> _fetchRestaurantHours(List<Restaurant> restaurants) async {
    final response = await http
      .get(Uri.parse("https://cs.furman.edu/~csdaemon/FUNow/restaurantHoursGet.php"));

    if (response.statusCode == 200) {
      // If the server did return a 200 OK response,
      // then parse the JSON.
      final hoursListJson = jsonDecode(response.body);
      for (var hoursJson in (hoursListJson["results"] as List<dynamic>)) {
        var hours = RestaurantHours.fromJson(hoursJson);
        restaurants.firstWhere((restaurant) => restaurant.id == hours.id).hoursList.add(hours);
      }
    } else {
      // If the server did not return a 200 OK response,
      // then throw an exception.
      throw Exception('Failed to load athletics events.');
    }
  }
}

class Restaurant {
  int id;
  String name;
  String? shortName;
  String location;
  LatLng mapLocation;
  int frequency;
  int busyness;
  String? url;
  List<RestaurantHours> hoursList = <RestaurantHours>[];
  Color? backgroundColor;

  Restaurant({
    required this.id,
    required this.name,
    this.shortName,
    required this.location,
    required this.mapLocation,
    required this.frequency,
    required this.busyness,
    this.url,
  });

  bool get isOpen {
    for(var hours in hoursList) {
      var now = DateTime.now();
      var today = Weekday.values[now.weekday - 1];
      if (hours.daysOfWeek.days[today] == true) {
        var currentTime = Duration(hours: now.hour, minutes: now.minute, seconds: now.second);
        if(hours.startTime != null && hours.endTime != null) {
          if (hours.startTime! < currentTime && hours.endTime! > currentTime) {
            return true;
          }
        }
      }
    }
    return false;
  }

  // TODO: this is the absolute worst
  String get imageUri {
    return "https://cs.furman.edu/~csdaemon/FUNow/appIcons/$name Icon.png";
  }

  ImageProvider get image {
    return NetworkImage(imageUri);
  }

  Future<void> _generateColor() async {
    var paletteGenerator = await PaletteGenerator.fromImageProvider(
      image,
      maximumColorCount: 1,
    );
    if (paletteGenerator.colors.toList().isNotEmpty) {
      var hslColor = HSLColor.fromColor(paletteGenerator.colors.toList()[0]);
      var newColor = HSLColor.fromAHSL(1, hslColor.hue, 40/100, 90/100);
      backgroundColor = newColor.toColor();
    }
  }

  factory Restaurant.fromJson(Map<String, dynamic> json) {
    return Restaurant(
        id: int.parse(json["id"]),
        name: json["fullname"],
        shortName: json["name"],
        location: json["location"],
        mapLocation: LatLng(double.parse(json["latitude"]), double.parse(json["longitude"])),
        frequency: int.parse(json["frequency"]),
        busyness: int.parse(json["busyness"]),
        url: (json["url"] != null && (json["url"] as String).isNotEmpty) ? json["url"] : null,
    );
  }
}

class RestaurantHours {
  int id;
  String? meal;
  Duration? startTime;
  Duration? endTime;
  DaysOfWeek daysOfWeek;
  int dayOrder;

  RestaurantHours({
    required this.id,
    this.meal,
    this.startTime,
    this.endTime,
    required this.daysOfWeek,
    required this.dayOrder,
  });

  static Duration _getDurationFromString(String s) {
    var timeComponents = s.split(":");
    var hours = int.parse(timeComponents[0]);
    var minutes = int.parse(timeComponents[1]);
    var seconds = int.parse(timeComponents[2]);

    return Duration(hours: hours, minutes: minutes, seconds: seconds);
  }

  factory RestaurantHours.fromJson(Map<String, dynamic> json) {
    return RestaurantHours(
      id: int.parse(json["id"]),
      meal: json["meal"],
      startTime: (json["start"] != null) ? _getDurationFromString(json["start"]) : null,
      endTime: (json["end"] != null) ? _getDurationFromString(json["end"]) : null,
      daysOfWeek: DaysOfWeek.parse(json["dayOfWeek"]),
      dayOrder: int.parse(json["dayOrder"]),
    );
  }
}
