import 'package:flutter/material.dart';
import 'package:furman_now/src/services/events/event.dart';
import 'package:furman_now/src/services/events/events_service.dart';
import 'package:intl/intl.dart';

void showEventsModal(BuildContext context, Event event) {
  showModalBottomSheet(
    context: context,
    isScrollControlled: true,
    backgroundColor: Colors.transparent,
    builder: (context) {
      return _EventModal(event: event);
    },
  );
}

class _EventModal extends StatefulWidget {
  final Event event;

  const _EventModal({
    required this.event,
    Key? key,
  }) : super(key: key);

  @override
  State<_EventModal> createState() => _EventModalState();
}

class _EventModalState extends State<_EventModal> {
  var childSize = Size.zero;
  BoxConstraints? _parentConstraints;
  final _key = GlobalKey();
  
  double _maxChildHeight = 0.75;

  void updateMaxHeight() {
    if (_parentConstraints != null) {
      var listHeight = _key.currentContext?.size?.height;
      var parentHeight = _parentConstraints?.maxHeight;
      if (listHeight != null && parentHeight != null) {
        var maxHeight = (listHeight + 50) / parentHeight;
        setState(() {
          _maxChildHeight = (maxHeight < 0.75) ? maxHeight : 0.75;
        });
      }
    }
  }

  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance
      .addPostFrameCallback((_) => updateMaxHeight());
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () => Navigator.of(context).pop(),
      child: Container(
        color: const Color.fromRGBO(0, 0, 0, 0.001),
        child: LayoutBuilder(
          builder: (BuildContext context, BoxConstraints constraints) {
            _parentConstraints = constraints;
            return GestureDetector(
              onTap: () {},
              child: DraggableScrollableSheet(
                  initialChildSize: _maxChildHeight,
                  minChildSize: 0.1,
                  maxChildSize: _maxChildHeight,
                  builder: (_, controller) {
                    return Container(
                      color: Colors.grey.shade100,
                      child: Column(
                        children: [
                          const Align(
                            alignment: Alignment.topRight,
                            child: CloseButton(),
                          ),
                          Flexible(
                            child: NotificationListener<OverscrollIndicatorNotification>(
                              onNotification: (OverscrollIndicatorNotification overscroll) {
                                overscroll.disallowIndicator();
                                return true;
                              },
                              child: _EventModalContent(
                                key: _key,
                                controller: controller,
                                event: widget.event,
                              ),
                            ),
                          ),
                        ],
                      ),
                    );
                  }
              ),
            );
          }
        ),
      ),
    );
  }
}

class _EventModalContent extends StatelessWidget {
  final ScrollController controller;
  final Event event;

  const _EventModalContent({
    required this.controller,
    required this.event,
    Key? key,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return ListView(
      shrinkWrap: true,
      padding: const EdgeInsets.only(left: 40, right: 40, bottom: 40),
      controller: controller,
      children: <Widget>[
        // Title
        Text(
          event.title,
          style: Theme.of(context).textTheme.headline1,
        ),
        const SizedBox(height: 10),
        // Location
        RichText(text: TextSpan(
          style: Theme.of(context).textTheme.subtitle1,
          children: [
            WidgetSpan(
              alignment: PlaceholderAlignment.middle,
              child: Padding(
                  padding: const EdgeInsets.only(right: 5.0),
                  child: Icon(
                      Icons.place_outlined, size: 24,
                      color: Colors.grey[500])
              ),
            ),
            TextSpan(text: event.location),
          ],
        )),
        const SizedBox(height: 5),
        if (event is ClpEvent) ...[
          // Organization
          RichText(text: TextSpan(
            style: Theme.of(context).textTheme.subtitle1,
            children: [
              WidgetSpan(
                alignment: PlaceholderAlignment.middle,
                child: Padding(
                    padding: const EdgeInsets.only(right: 5.0),
                    child: Icon(
                        Icons.group_outlined,
                        size: 24,
                        color: Colors.grey[500]
                    ),
                ),
              ),
              TextSpan(text: (event as ClpEvent).organization),
            ],
          )),
          const SizedBox(height: 5),
          // Time
          RichText(text: TextSpan(
            style: Theme.of(context).textTheme.subtitle1,
            children: [
              WidgetSpan(
                alignment: PlaceholderAlignment.middle,
                child: Padding(
                    padding: const EdgeInsets.only(
                        right: 5.0),
                    child: Icon(
                        Icons.access_time_outlined,
                        size: 24,
                        color: Colors.grey[500])
                ),
              ),
              WidgetSpan(
                  alignment: PlaceholderAlignment.top,
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment
                        .start,
                    children: [
                      Text(
                        DateFormat("EEEE, MMMM d")
                            .format(
                            (event as ClpEvent)
                                .startTime),
                        style: Theme
                            .of(context)
                            .textTheme
                            .subtitle1,
                      ),
                      Text(
                        "${DateFormat.jm().format(
                            (event as ClpEvent)
                                .startTime)} - ${DateFormat
                            .jm().format(
                            (event as ClpEvent)
                                .endTime)}",
                        style: Theme
                            .of(context)
                            .textTheme
                            .subtitle1,
                      ),
                    ],
                  )
              ),
            ],
          )),
          const SizedBox(height: 5),
          // Description
          Row(
            crossAxisAlignment: CrossAxisAlignment
                .start,
            children: [
              Padding(
                  padding: const EdgeInsets.only(
                      right: 5.0),
                  child: Icon(Icons.notes, size: 24,
                      color: Colors
                          .grey[500])
              ),
              Flexible(
                child: Text(
                  (event as ClpEvent)
                      .description,
                  style: Theme
                      .of(context)
                      .textTheme
                      .subtitle1,
                ),
              ),
            ],
          ),
        ]
      ],
    );
  }
}
